@@ -9,11 +9,11 @@ Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.49_01
+Version 1.51_01
 
 =cut
 
-our $VERSION = '1.49_01';
+our $VERSION = '1.51_01';
 
 =head1 SYNOPSIS
 
@@ -442,8 +442,17 @@ sub follow_link_ok {
 
 =head2 $mech->click_ok( $button[, $desc] )
 
-Clicks the button named by C<$button>.  An optional C<$desc> can
-be given for the test.
+=head2 $mech->click_ok( \@button-and-coordinates [, $desc ] )
+
+Clicks the button named by C<$button>.  An optional C<$desc> can be
+given for the test.
+
+    $mech->click_ok( 'continue', 'Clicking the "Continue" button' );
+
+Alternatively the first argument can be an arrayref with three elements:
+The name of the button and the X and Y coordinates of the button.
+
+    $mech->click_ok( [ 'continue', 12, 47 ], 'Clicking the "Continue" button' );
 
 =cut
 
@@ -452,12 +461,18 @@ sub click_ok {
     my $button = shift;
     my $desc   = shift;
 
-    my $response = $self->click( $button );
+    my $response;
+    if ( ref($button) eq 'ARRAY' ) {
+        $response = $self->click( $button->[0], $button->[1], $button->[2] );
+    }
+    else {
+        $response = $self->click( $button );
+    }
+
     if ( !$response ) {
         return $TB->ok( 0, $desc );
     }
 
-
     my $ok = $response->is_success;
 
     $ok = $self->_post_load_validation( $ok, $desc );
@@ -677,7 +692,7 @@ sub html_tidy_ok {
     my $ok;
 
     if ( $self->is_html ) {
-        $ok = $self->_lint_content_ok( $desc );
+        $ok = $self->_tidy_content_ok( $desc );
     }
     else {
         $ok = $TB->ok( 0, $desc );
@@ -708,7 +723,7 @@ sub _tidy_content_ok {
         $tidy = HTML::Tidy5->new();
     }
 
-    $tidy->parse( '', $self->content );
+    $tidy->parse( '', $self->content_for_tidy );
 
     my @messages = $tidy->messages;
     my $nmessages = @messages;
@@ -728,6 +743,24 @@ sub _tidy_content_ok {
 }
 
 
+=head2 $mech->content_for_tidy()
+
+This method is called by C<html_tidy_ok()> to get the content that should
+be validated by HTML::Tidy5. By default, this is just C<content()>,
+but subclasses can override it to modify the content before validation.
+
+This method should not change any state in the Mech object.  Specifically,
+it should not actually modify any of the actual content.
+
+=cut
+
+sub content_for_tidy {
+    my $self = shift;
+
+    return $self->content;
+}
+
+
 =head2 $mech->title_is( $str [, $desc ] )
 
 Tells if the title of the page is the given string.
@@ -1581,6 +1614,193 @@ sub scraped_id_like {
 }
 
 
+=head2 id_exists( $id [, $html] )
+
+Returns TRUE/FALSE if the given ID exists in the given HTML, or if none
+is provided, then the current page.
+
+=cut
+
+sub id_exists {
+    my $self = shift;
+    my $id = shift;
+
+    my $html;
+    if ( @_ ) {
+        $html = shift;
+        assert_nonref( $html, '$html passed in is a populated scalar' );
+    }
+    else {
+        if ( $self->ct() eq 'text/html' && defined $self->{content} ) {
+            $html = $self->{ content };
+        }
+    }
+
+    assert_nonblank( $html, 'We have HTML to look at' );
+
+    my $found = index( $html, " id=\"$id\"" );
+    if ( $found < 0 ) {
+        $found = index( $html, " id='$id'" );
+    }
+    return $found >= 0;
+}
+
+=head2 $agent->id_exists_ok( $id [, $msg] )
+
+Verifies there is an HTML element with ID C<$id> in the page.
+
+=cut
+
+sub id_exists_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my $id   = shift;
+    my $msg  = shift || ('ID "' . ($id || '') . '" should exist');
+
+    my $exists = $self->id_exists( $id, $self->content() );
+
+    return $TB->ok( $exists, $msg );
+}
+
+=head2 $agent->ids_exist_ok( \@ids [, $msg] )
+
+Verifies an HTML element exists with each ID in C<\@ids>.
+
+=cut
+
+sub ids_exist_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my $ids  = shift;
+    my $msg  = shift;
+
+    assert_arrayref( $ids );
+
+    my $subtest_name = 'ids_exist_ok( [' . join( ', ', @{$ids} ) . ']';
+    $subtest_name .= ", $msg" if defined $msg;
+    $subtest_name .= ' )';
+
+    return $TB->subtest(
+        $subtest_name,
+        sub {
+            $TB->plan( tests => scalar @{$ids} );
+
+            foreach my $id ( @$ids ) {
+                $self->id_exists_ok( $id );
+            }
+        }
+    );
+}
+
+=head2 $agent->lacks_id_ok( $id [, $msg] )
+
+Verifies there is NOT an HTML element with ID C<$id> in the page.
+
+=cut
+
+sub lacks_id_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my $id   = shift;
+    my $msg  = shift || ('ID "' . ($id || '') . '" should not exist');
+
+    assert_nonblank( $id );
+
+    my $exists = $self->id_exists( $id, $self->content() );
+
+    return $TB->ok( !$exists, $msg );
+}
+
+
+=head2 $agent->lacks_ids_ok( \@ids [, $msg] )
+
+Verifies there are no HTML elements with any of the ids given in C<\@ids>.
+
+=cut
+
+sub lacks_ids_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self = shift;
+    my $ids = shift;
+    my $msg = shift;
+
+    assert_arrayref( $ids );
+
+    my $subtest_name = 'lacks_id_ok( [' . join( ', ', @{$ids} ) . ']';
+    $subtest_name .= ", $msg" if defined $msg;
+    $subtest_name .= ' )';
+
+    return $TB->subtest(
+        $subtest_name,
+        sub {
+            $TB->plan( tests => scalar @{$ids} );
+
+            foreach my $id ( @$ids ) {
+                $self->lacks_id_ok( $id, "ID '" . ($id // '') . "' should not exist" );
+            }
+        }
+    );
+}
+
+
+=head2 $mech->button_exists( $button )
+
+Returns a boolean saying whether the submit C<$button> exists. Does not
+do a test. For that you want C<button_exists_ok> or C<lacks_button_ok>.
+
+=cut
+
+sub button_exists {
+    my $self   = shift;
+    my $button = shift;
+
+    my $input = $self->grep_inputs( {
+        type => qr/^submit$/,
+        name => qr/^$button$/
+    } );
+
+    return !!$input;
+}
+
+
+=head2 $mech->button_exists_ok( $button [, $msg] )
+
+Asserts that the button exists on the page.
+
+=cut
+
+sub button_exists_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self   = shift;
+    my $button = shift;
+    my $msg    = shift;
+
+    return $TB->ok( $self->button_exists( $button ), $msg );
+}
+
+
+=head2 $mech->lacks_button_ok( $button [, $msg] )
+
+Asserts that the button exists on the page.
+
+=cut
+
+sub lacks_button_ok {
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+
+    my $self   = shift;
+    my $button = shift;
+    my $msg    = shift;
+
+    return $TB->ok( !$self->button_exists( $button ), $msg );
+}
+
+
 =head1 METHODS: MISCELLANEOUS
 
 =head2 $mech->autolint( [$status] )
@@ -1980,10 +2200,11 @@ L<http://search.cpan.org/dist/Test-WWW-Mechanize>
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to
+@marderh,
 Eric A. Zarko,
-moznion,
+@moznion,
 Robert Stone,
-tynovsky,
+@tynovsky,
 Jerry Gay,
 Jonathan "Duke" Leto,
 Philip G. Potter,
